@@ -1,6 +1,7 @@
 """
 """
 import collections.abc
+import contextlib
 import copy
 import functools
 import pyparsing as pp
@@ -8,6 +9,7 @@ import re
 
 
 _marker = object()
+ANY = _marker
 
 
 class Match:
@@ -63,14 +65,23 @@ class Const(Op):
         return ( v for v in vals if self.value == v )
 
 
-class Integer(Const):
-    @property
-    def value(self):
-        return int(self.args[0])
+class Numeric(Const):
+    def is_int(self):
+        try:
+            return str(self.value) == str(int(self.value))
+        except (ValueError, TypeError):
+            return False
     def __repr__(self):
         return f'{self.value}'
 
 
+class NumericQuoted(Numeric):
+    def __repr__(self):
+        if self.is_int():
+            return super().__repr__()
+        return f"#'{str(self.value)}'"
+
+
 class Word(Const):
     def __repr__(self):
         return f'{self.value}'
@@ -159,6 +170,9 @@ def itemof(node, val):
 class Key(Op):
     @classmethod
     def concrete(cls, val):
+        import numbers
+        if isinstance(val, numbers.Number):
+            return cls(NumericQuoted(val))
         return cls(Word(val))
     @property
     def op(self):
@@ -195,7 +209,7 @@ class Key(Op):
         return None if val is _marker else Match(val)
 
     def update(self, node, key, val):
-        node[key] = val
+        node[key] = self.default() if val is ANY else val
         return node
     def upsert(self, node, val):
         if not self.is_pattern():
@@ -209,19 +223,18 @@ class Key(Op):
         return node
     def remove(self, node, val):
         for k,v in self.items(node):
-            if v == val:
+            if val is ANY or v == val:
                 return self.pop(node, k)
         return node
-    def clear(self, node):
-        for k in list(self.keys(node)):
-            self.pop(node, k)
-        return node
 
 
 class Slot(Key):
     @classmethod
     def concrete(cls, val):
-        return cls(Integer(val) if isinstance(val, int) else String(val))
+        import numbers
+        if isinstance(val, numbers.Number):
+            return cls(Numeric(val))
+        return String(val)
     def __repr__(self):
         return f'[{self.op}]'
     def operator(self, top=False):
@@ -237,7 +250,7 @@ class Slot(Key):
         except (KeyError, IndexError):
             return ()
     def default(self):
-        if isinstance(self.op, Integer):
+        if isinstance(self.op, Numeric) and self.op.is_int():
             return []
         return super().default()
 
@@ -250,7 +263,7 @@ class Slot(Key):
         if node[key] == val:
             return node
         try:
-            node[key] = val
+            node[key] = self.default() if val is ANY else val
         except TypeError:
             node = node[:key] + itemof(node, val) + node[key+1:]
         return node
@@ -267,6 +280,15 @@ class Slot(Key):
     def remove(self, node, val):
         if hasattr(node, 'keys'):
             return super().remove(node, val)
+        if val is ANY:
+            if hasattr(node, 'pop'):
+                popped = 0
+                for k in list(self.keys(node)):
+                    node.pop(k - popped)
+                    popped += 1
+                return node
+            keys = tuple(self.keys(node))
+            return node.__class__(v for i,v in enumerated(node) if i not in keys)
         if hasattr(node, 'remove'):
             try:
                 node.remove(val)
@@ -279,17 +301,6 @@ class Slot(Key):
         except ValueError:
             pass
         return node
-    def clear(self, node):
-        if hasattr(node, 'keys'):
-            return super().clear(node)
-        if hasattr(node, 'pop'):
-            popped = 0
-            for k in list(self.keys(node)):
-                node.pop(k - popped)
-                popped += 1
-            return node
-        keys = tuple(self.keys(node))
-        return node.__class__(v for i,v in enumerated(node) if i not in keys)
 
 
 class SlotSpecial(Slot):
@@ -320,15 +331,12 @@ class SlotSpecial(Slot):
     def upsert(self, node, val):
         return self.update(node, self.op.value, val)
 
-
     def pop(self, node, key):
         if isinstance(key, str):
             return None
         return node
     def remove(self, node, val):
         return node
-    def clear(self, node):
-        return node
 
 
 class Slice(Op):
@@ -407,7 +415,7 @@ class Slice(Op):
         if node[key] == val:
             return node
         try:
-            node[key] = val
+            node[key] = self.default() if val is ANY else val
             return node
         except TypeError:
             pass
@@ -427,7 +435,6 @@ class Slice(Op):
     def upsert(self, node, val):
         return self.update(node, self.slice(node), val)
 
-
     def pop(self, node, key):
         try:
             del node[key]
@@ -440,12 +447,35 @@ class Slice(Op):
             return node.__class__().join(iterable)
         return node.__class__(iterable)
     def remove(self, node, val):
+        if val is ANY:
+            return self.pop(self, node, self.slice(node))
         key = self.slice(node)
         if node[key] == val:
             return self.pop(self, node, key)
         return node
-    def clear(self, node):
-        return self.pop(self, node, self.slice(node))
+
+
+class Invert(Op):
+    @classmethod
+    def concrete(cls, val):
+        return cls(val)
+    def __repr__(self):
+        return '-'
+    def is_pattern(self):
+        return False
+    @property
+    def op(self):
+        return None
+    def operator(self, top=False):
+        return '-'
+    def match(self, op, specials=False):
+        return Match('-') if isinstance(op, Invert) else None
+    def items(self, node):
+        yield ('-', node)
+    def keys(self, node):
+        yield '-'
+    def values(self, node):
+        yield node
 
 
 #
@@ -470,7 +500,13 @@ class Dotted:
         self.ops = tuple(results['ops'])
         self.transforms = tuple( tuple(r) for r in results.get('transforms', ()) )
     def assemble(self, start=0):
-        return ''.join(op.operator(idx==0) for idx,op in enumerate(self.ops[start:]))
+        def _gen():
+            top = True
+            for op in self.ops[start:]:
+                yield op.operator(top)
+                if not isinstance(op, Invert):
+                    top = False
+        return ''.join(_gen())
     def __repr__(self):
         return f'{self.__class__.__name__}({list(self.ops)}, {list(self.transforms)})'
     def __hash__(self):
@@ -523,6 +559,9 @@ def build(ops, node, deepcopy=True):
 
 def gets(ops, node):
     cur, *ops = ops
+    if isinstance(cur, Invert):
+        yield from gets(ops, node)
+        return
     values = cur.values(node)
     if not ops:
         yield from values
@@ -535,6 +574,8 @@ def updates(ops, node, val, has_defaults=False):
     cur, *ops = ops
     if not ops:
         return cur.upsert(node, val)
+    if isinstance(cur, Invert):
+        return removes(ops, node, val)
     if cur.is_empty(node) and not has_defaults:
         built = updates(ops, build_default(ops), val, True)
         return cur.upsert(node, built)
@@ -543,10 +584,12 @@ def updates(ops, node, val, has_defaults=False):
     return node
 
 
-def removes(ops, node, val=_marker):
+def removes(ops, node, val=ANY):
     cur, *ops = ops
     if not ops:
-        return cur.clear(node) if val is _marker else cur.remove(node, val)
+        return cur.remove(node, val)
+    if isinstance(cur, Invert):
+        return updates(ops, node, val)
     for k,v in cur.items(node):
         node = cur.update(node, k, removes(ops, v, val))
     return node
